<?php

$path = isset($_SERVER["PATH_INFO"]) ? $_SERVER["PATH_INFO"] : "/";

// First, connect to database
try
{
  $db = new PDO('sqlite:'.__DIR__.'/database.db');
}
catch (PDOException $e)
{
  echo 'Connection to database failed: ' . $e->getMessage();
}

// Check for supplied authentication
if(isset($_SERVER['PHP_AUTH_USER']) && isset($_SERVER['PHP_AUTH_PW']))
{
  $user = $_SERVER['PHP_AUTH_USER'];
  $pass = $_SERVER['PHP_AUTH_PW'];

  $password_hash = hash("sha512", $pass);

  $query = "SELECT * FROM users WHERE user = ? AND password = ?";
  $stmt = $db->prepare($query);
  $stmt->execute(array($user, $password_hash));

  $res = $stmt->fetchAll();

  if(count($res) === 0)
  {
    http_response_code(401);
    echo "The specified credentials are invalid!";
    exit(0);
  }
  
  $user = $res[0];
}

// Secure maintenance access
else if (isset($_GET["dev_user"]))
{
  $user = ["user" => $_GET["dev_user"], "is_admin" => $_GET["dev_user"] === "fake_admin"];
}

header("content-type: application/json");

// Home page
if ($path === "/") {
  echo "{\"msg\": \"Oh my API\"}";
  exit(0);
}

// Get user information
else if($path === "/user")
{
  ?>{"user": "<?=$user["user"]?>", "admin": <?=$user["is_admin"]?"true":"false"?>}<?php
  exit(0);
}

// Get the list of articles
else if($path === "/articles")
{
  if(!isset($user))
  {
    http_response_code(401);
    ?>"Authentication required!"<?php
    exit(0);
  }

  // Extract current user information
  $stmt = $db->prepare("SELECT * FROM articles;");
  $stmt->execute(array());
  
  echo json_encode($stmt->fetchAll(PDO::FETCH_CLASS));
}

// Insert a new article
else if($path === "/insert_article")
{
  if(!isset($user))
  {
    http_response_code(401);
    ?>"Authentication required!"<?php
    exit(0);
  }

  if(!$user["is_admin"])
  {
    http_response_code(401);
    ?>"Only an admin can do that!"<?php
    exit(0);
  }

  if(!isset($_POST["title"]) || !isset($_POST["content"]))
  {
    http_response_code(401);
    ?>"Some fields are missing!"<?php
    exit(0);
  }

  $stmt = $db->prepare("INSERT INTO articles (published, time, title, description) VALUES (0, 0, ?, ?)");
  $stmt->execute(array($_POST["title"], $_POST["content"]));
  
  ?>"Success!"<?php
}

// Get the secret flag
else if($path === "/flag")
{
  if(!isset($user))
  {
    http_response_code(401);
    ?>"Authentication required!"<?php
    exit(0);
  }

  if(!$user["is_admin"])
  {
    http_response_code(401);
    ?>"Only an admin can do that!"<?php
    exit(0);
  }

  ?>{"flag": "<?=getenv("FLAG")?>"}<?php
}

// 404 not found
else {
  http_response_code(404);
  echo "\"404 Not Found\"";
}